"use strict";
var intersection = require('lodash/intersection');
function buildModelDescriptor(spec, options) {
    var primaryAttribute = spec.attributeArray.filter(function (a) { return a.primary; })[0];
    var sequelizeModel = (options && options.sequelizeModel && options.sequelizeModel.sequelizeModel) || undefined;
    var resolver = (options && options.resolver && options.resolver.resolver) || undefined;
    var synchronizer = (options && options.synchronizer && options.synchronizer.synchronizer) || undefined;
    var aliasModelMap = spec.relationArray.reduce(function (m, r) {
        m[r.alias] = r.model;
        return m;
    }, {});
    var aliasKeyMap = spec.relationArray.reduce(function (m, r) {
        m[r.alias] = r.key;
        return m;
    }, {});
    var allowedAttributes = (options && options.allowedAttributes) || spec.attributeArray.map(function (a) { return a.name; });
    var allowedInsertAttributes = (options && options.allowedInsertAttributes)
        || spec.attributeArray.filter(function (a) { return a.insertable; }).map(function (a) { return a.name; });
    var allowedUpdateAttributes = (options && options.allowedUpdateAttributes)
        || spec.attributeArray.filter(function (a) { return a.updatable; }).map(function (a) { return a.name; });
    var defaultOrder = (options && options.defaultOrder)
        || (primaryAttribute && [primaryAttribute.name, 'ASC'])
        || undefined;
    function sanitizeAttributes(attributes) {
        var allowedAttributes = options && options.allowedAttributes;
        if (!Array.isArray(allowedAttributes)) {
            return null;
        }
        if (Array.isArray(attributes)) {
            return intersection(attributes, allowedAttributes);
        }
        return allowedAttributes;
    }
    function initialize(pool) {
        if (options && options.sequelizeModel) {
            options.sequelizeModel.initialize(pool);
        }
        if (options && options.resolver) {
            options.resolver.initialize(pool);
        }
        if (options && options.synchronizer) {
            options.synchronizer.initialize(pool);
        }
    }
    return {
        name: spec.name,
        primaryKey: primaryAttribute ? primaryAttribute.name : undefined,
        sequelizeModel: sequelizeModel,
        resolver: resolver,
        synchronizer: synchronizer,
        aliasModelMap: aliasModelMap,
        aliasKeyMap: aliasKeyMap,
        allowedAttributes: allowedAttributes,
        allowedInsertAttributes: allowedInsertAttributes,
        allowedUpdateAttributes: allowedUpdateAttributes,
        defaultOrder: defaultOrder,
        fieldWhereBuilders: (options && options.fieldWhereBuilders) || undefined,
        sanitizeAttributes: sanitizeAttributes,
        initialize: initialize
    };
}
exports.buildModelDescriptor = buildModelDescriptor;
