"use strict";
var Promise = require('bluebird');
var fetch_1 = require('./fetch');
function buildFetcherCreator(endPointUrl) {
    function buildFetcher() {
        var id = 0;
        var feched = false;
        var requests = [];
        function addId(query) {
            id += 1;
            query.queryId = id;
            return query;
        }
        function add(query) {
            return new Promise(function (resolve, reject) {
                requests.push({ query: addId(query), resolve: resolve, reject: reject });
            });
        }
        function fetch() {
            if (feched) {
                throw new Error('fech cannot be called on the same object more than once');
            }
            if (requests.length === 0) {
                return Promise.resolve();
            }
            feched = true;
            return fetch_1.fetch({
                url: endPointUrl,
                method: 'POST',
                body: requests.map(function (r) { return r.query; })
            }).then(function (data) {
                var requestsByQueryId = {};
                requests.forEach(function (req) { return requestsByQueryId[req.query.queryId] = req; });
                data.forEach(function (item) {
                    var request = requestsByQueryId[item.queryId];
                    request.resolve(item);
                });
            }).catch(function (error) {
                requests.forEach(function (req) { return req.reject(error); });
            });
        }
        return { add: add, fetch: fetch };
    }
    function buildSingleShotFetcher(harness) {
        var fetcher = buildFetcher();
        harness(fetcher);
        return fetcher.fetch();
    }
    return { buildFetcher: buildFetcher, buildSingleShotFetcher: buildSingleShotFetcher };
}
exports.buildFetcherCreator = buildFetcherCreator;
