"use strict";
var Sequelize = require('sequelize');
function buildSequelizeModel(spec, sequelize, options) {
    var attributes = {};
    spec.attributeArray.forEach(function (attrib) {
        return attributes[attrib.name] = {
            type: datatypeToSequelizeType(attrib.type),
            allowNull: !attrib.required,
            autoIncrement: attrib.primary && !attrib.manual,
            defaultValue: attrib.defaultValue,
            field: (options && options.attributeToFieldMap && options.attributeToFieldMap[attrib.name]) || undefined,
            primaryKey: attrib.primary,
            values: attrib.values ? attrib.values.map(function (v) { return v.name; }) : undefined,
            unique: !attrib.primary && attrib.unique
        };
    });
    var defineOptions = {
        tableName: (options && options.tableName) || spec.name
    };
    if (spec.indexes.unique) {
        defineOptions.indexes = spec.indexes.unique.map(function (fields) { return ({ unique: true, fields: fields }); });
    }
    var model = sequelize.define(spec.name, attributes, defineOptions);
    if (options && options.createDatabase) {
        options.createDatabase(model);
    }
    function initialize(pool) {
        spec.nonVirtualRelations.superclass.forEach(function (superclass) { return model.belongsTo(pool.models[superclass.model].sequelizeModel, {
            as: superclass.alias,
            foreignKey: superclass.key,
            onDelete: superclass.onDelete,
            onUpdate: superclass.onUpdate
        }); });
        spec.nonVirtualRelations.subclass.forEach(function (subclass) { return model.hasOne(pool.models[subclass.model].sequelizeModel, {
            as: subclass.alias,
            foreignKey: subclass.key,
            onDelete: subclass.onDelete,
            onUpdate: subclass.onUpdate
        }); });
        spec.nonVirtualRelations.manyToMany.forEach(function (manyToMany) { return model.belongsToMany(pool.models[manyToMany.model].sequelizeModel, {
            through: pool.models[manyToMany.through].sequelizeModel,
            as: manyToMany.alias,
            foreignKey: manyToMany.key,
            onDelete: manyToMany.onDelete,
            onUpdate: manyToMany.onUpdate
        }); });
        spec.nonVirtualRelations.master.forEach(function (master) { return model.belongsTo(pool.models[master.model].sequelizeModel, {
            as: master.alias,
            foreignKey: master.key,
            onDelete: master.onDelete,
            onUpdate: master.onUpdate
        }); });
        spec.nonVirtualRelations.detail.forEach(function (detail) { return model.hasMany(pool.models[detail.model].sequelizeModel, {
            as: detail.alias,
            foreignKey: detail.key,
            onDelete: detail.onDelete,
            onUpdate: detail.onUpdate
        }); });
        spec.nonVirtualRelations.recursive.forEach(function (recursive) { return model.hasOne(pool.models[recursive.model].sequelizeModel, {
            as: recursive.alias,
            foreignKey: recursive.key,
            onDelete: recursive.onDelete,
            onUpdate: recursive.onUpdate
        }); });
    }
    return { sequelizeModel: model, initialize: initialize };
}
exports.buildSequelizeModel = buildSequelizeModel;
function datatypeToSequelizeType(datatype) {
    switch (datatype) {
        case 'INTEGER':
            return Sequelize.INTEGER;
        case 'STRING':
            return Sequelize.TEXT;
        case 'OBJECT':
            return Sequelize.JSONB;
        case 'ENUM':
            return Sequelize.ENUM;
        case 'BOOLEAN':
            return Sequelize.BOOLEAN;
        case 'DOUBLE':
            return Sequelize.DOUBLE;
    }
}
